## source model_fit.R for the results
source('./model_fit.R')
Sys.setlocale("LC_TIME", "English") 

###########################################################
## curves for the selected states, before and after smoothing
somdate <- nmfgap[, (startdate - 1):(startdate + gap*nperiod - 2)]
somdate <- data.frame(somdate)
names(somdate) <- datelist[(startdate):(startdate+gap*nperiod - 1)]
somdate$Province_State <- dat_confirmed_sum$Province_State

somdate2 <- fdat[, (startdate-1):(startdate+gap*nperiod - 2)]
somdate2 <- data.frame(somdate2)
names(somdate2) <- datelist[(startdate):(startdate + gap*nperiod - 1)]
somdate2$Province_State <- dat_confirmed_sum$Province_State

curveplot <- function(state){
  day1 <- ymd('2020-03-16')
  xline <- day1 + 7*seq(1, nperiod + 1)
  s2 <- somdate2 %>% filter(Province_State == tolower(state)) %>%
    gather(days, gap,1:(ncol(somdate2) - 1)) %>% 
    mutate(date = ymd(days))
  pl <- somdate %>% filter(Province_State == tolower(state)) %>%
    gather(days, gap, 1:(ncol(somdate) - 1)) %>%
    mutate(date = ymd(days)) %>% 
    ggplot(aes(x = date, y = gap, color = Province_State)) + 
    geom_line(color = 1, size = 0.3) + 
    geom_vline(xintercept = xline, linetype = 2, color = 2, 
               size = 0.3, alpha = 0.3) + 
    xlab(c("")) + ylab(c("")) + 
    ggtitle(state) + 
    geom_line(data = s2, aes(x = date, y = gap), 
              color = 4, size = 0.6) + 
    theme_bw() + 
    theme(text = element_text(size = 12), 
          plot.title = element_text(hjust = 0.5))
  return(pl)
  
}

ny <- curveplot('New York')
nc <- curveplot('North Carolina')
la <- curveplot("Louisiana")
ma <- curveplot('Massachusetts')

g <- grid.arrange(ny, nc, la, ma, ncol = 2)
ggsave(filename = '../manuscript/periodic2.pdf', 
       g, 
       width = 6.2, height = 6.5, 
       units = 'in')

###########################################################
## get ARI for successive study periods
crand <- NULL
for(i in 2:nperiod){
  face <- dist(clusters[[i]])
  ccompare <- cluster.stats(d = face,  
                            clustering=groups[, i - 1],  
                            alt.clustering = groups[, i])
  crand <- c(crand, ccompare$corrected.rand)
}

i <- 1:(nperiod-1)
crand <- data.frame(cutdate = 
                      format(as.Date
                             (datelist[startdate + gap*i - 1]), 
                             "%m/%d"), 
                    ARI = crand)

ggplot(crand, aes(x = cutdate, y = ARI, group = 1)) + 
  geom_line() + 
  geom_point() + 
  theme_bw() + 
  theme(axis.text.x = element_text(angle = 90, hjust=1), 
        text = element_text(size = 9), 
        strip.text = element_text(size = 9), 
        plot.title = element_text(hjust = 0.5), 
        plot.margin = margin(1,1,1,1, "mm"),  
        axis.title = element_blank()) +
  xlab(c("")) + ylab(c("")) + 
  ggtitle('ARI')
ggsave(filename = '../manuscript/ari.pdf', 
       width = 5, height = 2.2, 
       units = 'in')

## ARI between T10 and T18
ccompare <- cluster.stats(d = face,  
                          clustering = groups[, 10],  
                          alt.clustering = groups[, 18])
ccompare$corrected.rand

###########################################################
## generate us map for 2 time periods
cutdates <- as.matrix(dat_confirmed_sum %>% 
                        select("7/25/20", "5/30/20"))
cutdates <- wmatrix %*% cutdates
cutdates <- data.frame(p725 = cutdates[, 1],
                       p530 = cutdates[, 2],
                       g725 = groups[, 18],
                       g530 = groups[, 10])

## rank each group by averaged confirmed cases
a725 <- cutdates %>% group_by(g725) %>% 
  summarise(avg = mean(p725))
a725$cluster <- rank(a725$avg)
a530 <- cutdates %>% group_by(g530) %>% 
  summarise(avg = mean(p530))
a530$cluster <- rank(a530$avg)
c725 <- cutdates %>% left_join(a725)
c530 <- cutdates %>% left_join(a530)

plotmat <- data.frame(
  Province_State = rep(dat_confirmed_sum$Province_State, 2),
  g = c(groups[, 18], groups[, 10]), 
  period = c(rep('03/22 to 07/25', nrow(dat_confirmed_sum)), 
  rep('03/22 to 05/30', nrow(dat_confirmed_sum))))
plotmat$cluster <- c(c725$cluster, c530$cluster)
plotmat$region <- tolower(plotmat$Province_State)
plotmat <- plotmat %>% left_join(states_map,by = "region")
plotmat$period <- factor(
  plotmat$period, 
  levels = c('03/22 to 07/25', '03/22 to 05/30'))

## selected color for each group
color1 <- c("#DEEBF7", "#9ECAE1", "#6BAED6", "#4292C6", 
         "#2171B5", "#084594", '#000033')

ggplot() + 
  geom_polygon(
    data = plotmat, 
    aes(long, lat, group=group, 
        fill = as.factor(cluster)), 
    color = "gray70", size=0.01) +
  scale_fill_manual(values = color1) +
  coord_map(projection = "albers", lat0 = 37, lat1 = 42, 
            xlim = c(-118, -75), 
            ylim = c(25.1, 49.4)) +
  facet_wrap(~period,ncol = 2) +
  theme_bw() +
  theme(legend.position = "none",
        text = element_text(size = 8), 
        strip.text.x = element_text(size = 8), 
        axis.title = element_blank(),
        axis.text.x = element_blank(),
        axis.text.y = element_blank(),
        axis.ticks.x = element_blank(),
        axis.ticks.y = element_blank(),
        plot.margin = margin(2, 2, 0, 2, "mm")) +
   xlab(c(""))+ylab(c(""))

ggsave(filename = '../manuscript/map2.pdf', 
       width = 5, height = 2, 
       units = 'in')

###########################################################
## mean curves for T18
plotdat <- data.frame(fdat)
names(plotdat) <- 
  names(dat_confirmed_sum)[3:ncol(dat_confirmed_sum)]
plotdat <- plotdat[,c((startdate - 1):(startdate + gap*nperiod-2))]
plotdat$Province_State <- dat_confirmed_sum$Province_State


c1 <- dat_confirmed_sum$Province_State[groups[, 18] == 7]
c2 <- dat_confirmed_sum$Province_State[groups[, 18] == 3]
c3 <- dat_confirmed_sum$Province_State[groups[, 18] == 2]
c4 <- dat_confirmed_sum$Province_State[groups[, 18] == 6]
c5 <- dat_confirmed_sum$Province_State[groups[, 18] == 5]
c6 <- dat_confirmed_sum$Province_State[groups[, 18] == 1]
c7 <- dat_confirmed_sum$Province_State[groups[, 18] == 4]


clusters <- data.frame(
  Province_State = c(c1, c2, c3, c4, c5, c6, c7),
  cluster = c(rep('Cluster A', length(c1)), 
  rep('Cluster B', length(c2)), 
  rep('Cluster C', length(c3)), 
  rep('Cluster D', length(c4)),
  rep('Cluster E', length(c5)), 
  rep('Cluster F', length(c6)), 
  rep('Cluster G', length(c7))))

plotdat <- plotdat %>% left_join(clusters)
meancurve1 <- plotdat %>% filter(cluster == 'Cluster A') %>% 
  select(-Province_State,-cluster) %>%
  summarise_all(funs(mean)) 
meancurve1$Province_State <- 'means'
meancurve1$cluster <- 'Cluster A'

meancurve2 <- plotdat %>% 
  filter(cluster == 'Cluster B') %>% 
  select(-Province_State, -cluster) %>%
  summarise_all(funs(mean)) 
meancurve2$Province_State <- 'means'
meancurve2$cluster <- 'Cluster B'

meancurve3 <- plotdat %>% 
  filter(cluster == 'Cluster C') %>% 
  select(-Province_State, -cluster) %>%
  summarise_all(funs(mean)) 
meancurve3$Province_State <- 'means'
meancurve3$cluster <- 'Cluster C'
transplot <- plotdat %>% 
  gather(days, gap, 1:(ncol(plotdat)-2)) %>%
  mutate(date = mdy(days)) 

meancurve6 <- plotdat %>% 
  filter(cluster == 'Cluster F') %>% 
  select(-Province_State, -cluster) %>%
  summarise_all(funs(mean)) 
meancurve6$Province_State <- 'means'
meancurve6$cluster <- 'Cluster F'

meancurve7 <- plotdat %>% 
  filter(cluster == 'Cluster G') %>% 
  select(-Province_State, -cluster) %>%
  summarise_all(funs(mean)) 
meancurve7$Province_State <- 'means'
meancurve7$cluster <- 'Cluster G'

plotdat <- rbind(plotdat, meancurve1, 
                meancurve2, meancurve3, 
                meancurve6, meancurve7)

clustermeans <- plotdat[50:54,] %>% 
  gather(days, gap, 1:(ncol(plotdat)-2)) %>%
  mutate(date = mdy(days))

plotdat %>% 
  filter(Province_State != 'means') %>% 
  gather(days, gap, 1:(ncol(plotdat) - 2)) %>%
  mutate(date = mdy(days)) %>% 
  ggplot(aes(x = date, y = gap, color = Province_State)) + 
  geom_line(size = 0.3, alpha = 0.5) + 
  theme_bw() + 
  theme(legend.position = "none") + 
  ylab(c("Scaled Daily confirmed counts")) + 
  xlab(c("Date")) + 
  geom_line(data = clustermeans, 
           aes(x = date, y = gap), color = 1, size = 0.5) + 
  facet_wrap(~cluster, ncol = 2) + 
  theme(text = element_text(size = 8) , 
        strip.text = element_text(
          size=8, lineheight=0.1, 
          margin = margin(.05, 0, .05, 0, "cm")),
        plot.title = element_text(hjust = 0.5), 
        axis.title.y = element_text(vjust = 2))
ggsave(filename = '../manuscript/meanlines.pdf', 
       width = 4.5, height = 5.3, 
       units = 'in')

###########################################################
## make plots for the basis (T18)
r12 <- 12 # NMF rank of T18
nmfdat <-  fdat[,c((startdate - 1):(startdate + gap*nperiod - 2))]
w <- matrix(1, ncol(nmfdat), nrow(nmfdat))
d <- ncol(nmfdat)
resfit1 <- nmf(t(nmfdat), rank = 12, method = 'ls-nmf', 
               seed = 'nndsvd', weight = w)
base <- basis(resfit1)
basis <- NULL
type <- NULL
for(i in 1:r12){
  basis <- c(basis, base[, i])
  type <- c(type, rep(paste0('basis ', i), d))
}
base <- data.frame(
  date = rep(datelist[(startdate):(startdate + gap*nperiod - 1)], r12), 
  basis, type)
levels1 <- NULL
for(i in 1:r12){
  levels1 <- c(levels1, rep(paste0('basis ', i), 1))
}

base$type <- factor(
  base$type, 
  levels = levels1)

ggplot(base, aes(x = date, y = basis)) + 
  geom_line() + facet_wrap(~type, ncol = 3) + 
  theme_bw() + 
  theme(text = element_text(size = 8), 
        strip.text = element_text(
          size = 8, 
          lineheight = 0.01, 
          margin = margin(.05, 0, .05, 0, "cm")),
        plot.title = element_text(hjust = 0.5),
        axis.title.y = element_text(vjust= 2)) + 
  ylab(c("")) + 
  xlab(c(""))
ggsave(filename = '../manuscript/basis2.pdf', 
       width = 4.5, height = 5.3, 
       units = 'in')

###########################################################
## mean curves for T10 (not presented in the table, maybe useful for comparison with T18)
plotdat <- data.frame(fdat)
names(plotdat) <- names(dat_confirmed_sum)[3:ncol(dat_confirmed_sum)]
plotdat <- plotdat[, c((startdate - 1):(startdate + gap*10 - 2))]
plotdat$Province_State <- dat_confirmed_sum$Province_State

c1 <- dat_confirmed_sum$Province_State[groups[, 10] == 5]
c2 <- dat_confirmed_sum$Province_State[groups[, 10] == 4]
c3 <- dat_confirmed_sum$Province_State[groups[, 10] == 1]
c4 <- dat_confirmed_sum$Province_State[groups[, 10] == 2]
c5 <- dat_confirmed_sum$Province_State[groups[, 10] == 6]
c6 <- dat_confirmed_sum$Province_State[groups[, 10] == 3]


clusters <- data.frame(
  Province_State = c(c1, c2, c3, c4, c5, c6), 
  cluster = c(rep('Cluster A', length(c1)), 
              rep('Cluster B', length(c2)), 
              rep('Cluster C', length(c3)), 
              rep('Cluster D', length(c4)), 
              rep('Cluster E', length(c5)), 
              rep('Cluster F', length(c6))))
plotdat <- plotdat %>% left_join(clusters)

meancurve3 <- plotdat %>% 
  filter(cluster == 'Cluster C') %>% 
  select(-Province_State,-cluster) %>%
  summarise_all(funs(mean)) 
meancurve3$Province_State <- 'means'
meancurve3$cluster <- 'Cluster C'
transplot <- plotdat %>% 
  gather(days, gap,1:(ncol(plotdat)-2)) %>%
  mutate(date = mdy(days)) 

meancurve4 <- plotdat %>% 
  filter(cluster == 'Cluster D') %>% 
  select(-Province_State, -cluster) %>%
  summarise_all(funs(mean)) 
meancurve4$Province_State <- 'means'
meancurve4$cluster <- 'Cluster D'
transplot <- plotdat %>% 
  gather(days,gap, 1:(ncol(plotdat)-2)) %>%
  mutate(date = mdy(days))

meancurve5 <- plotdat %>% 
  filter(cluster == 'Cluster E') %>% 
  select(-Province_State, -cluster) %>%
  summarise_all(funs(mean)) 
meancurve5$Province_State <- 'means'
meancurve5$cluster <- 'Cluster E'
transplot <- plotdat %>% 
  gather(days,gap, 1:(ncol(plotdat)-2)) %>%
  mutate(date = mdy(days))

meancurve6 <- plotdat %>% 
  filter(cluster == 'Cluster F') %>% 
  select(-Province_State, -cluster) %>%
  summarise_all(funs(mean)) 
meancurve6$Province_State <- 'means'
meancurve6$cluster <- 'Cluster F'

plotdat <- rbind(plotdat, meancurve3, meancurve4, 
                meancurve5, meancurve6)

clustermeans <- plotdat[50:53,] %>% 
  gather(days, gap,1:(ncol(plotdat)-2)) %>%
  mutate(date = mdy(days))

plotdat %>% filter(Province_State != 'means') %>% 
  gather(days, gap,1:(ncol(plotdat) - 2)) %>%
  mutate(date = mdy(days)) %>% 
  ggplot(aes(x = date, y = gap, color = Province_State)) + 
  geom_line(size = 0.3, alpha = 0.5) + 
  theme_bw() + 
  theme(legend.position = "none") + 
  ylab(c("Scaled Daily confirmed counts")) + 
  xlab(c("Date")) + 
  geom_line(data = clustermeans,  
            aes(x = date, y = gap), color = 1, size = 0.5) + 
  facet_wrap(~cluster, ncol = 2) + 
  theme(text = element_text(size = 8), 
        strip.text = element_text(
          size = 8, lineheight = 0.1, 
          margin = margin(.05, 0, .05, 0, "cm")),
        plot.title = element_text(hjust = 0.5),
        axis.title.y = element_text(vjust = 2))
 
